@@ -1,17 +1,17 @@
 # The MIT License
-# 
+#
 # Copyright (c) 2010 Jeffrey Jenkins
-# 
+#
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
 # in the Software without restriction, including without limitation the rights
 # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
-# 
+#
 # The above copyright notice and this permission notice shall be included in
 # all copies or substantial portions of the Software.
-# 
+#
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -20,6 +20,9 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+from __future__ import print_function
+from mongoalchemy.py3compat import *
+
 from mongoalchemy.exceptions import BadValueException
 
 class BadQueryException(Exception):
@@ -43,7 +46,7 @@ class FreeFormField(object):
         return value
     def subfields(self):
         return FreeFormField(name=None)
-    def is_valid_wrap(*args): 
+    def is_valid_wrap(*args):
         return True
     is_valid_unwrap = is_valid_wrap
     __contains__ = is_valid_wrap
@@ -62,7 +65,7 @@ class FreeFormDoc(object):
     def transform_incoming(self, obj, session):
         return obj
     def get_indexes(self):
-        return []        
+        return []
     mongo_id = FreeFormField(name='_id')
 
 Q = FreeFormDoc('')
@@ -74,7 +77,7 @@ class QueryField(object):
         self.__cached_id_value = None
         self.__matched_index = False
         self.__fields_expr = True
-    
+
     @property
     def fields_expression(self):
         return flatten(self.__fields_expr)
@@ -84,44 +87,44 @@ class QueryField(object):
         if self.__cached_id_value is None:
             self.__cached_id_value = str(self)
         return self.__cached_id_value
-    
+
     def _get_parent(self):
         return self.__parent
 
     def get_type(self):
         ''' Returns the underlying :class:`mongoalchemy.fields.Field` '''
         return self.__type
-    
+
     def matched_index(self):
         ''' Represents the matched array index on a query with objects inside
             of a list.  In the MongoDB docs, this is the ``$`` operator '''
         self.__matched_index = True
         return self
-    
+
     def __getattr__(self, name):
         if not self.__type.no_real_attributes and hasattr(self.__type, name):
             return getattr(self.__type, name)
-        
+
         if not self.__type.has_subfields:
             raise AttributeError(name)
-        
+
         fields = self.__type.subfields()
         if name not in fields:
             raise BadQueryException('%s is not a field in %s' % (name, self.__type.sub_type()))
         return QueryField(fields[name], parent=self)
-    
+
     def get_absolute_name(self):
         """ Returns the full dotted name of this field """
         res = []
         current = self
-        
+
         while type(current) != type(None):
             if current.__matched_index:
                 res.append('$')
             res.append(current.get_type().db_field)
             current = current._get_parent()
         return '.'.join(reversed(res))
-    
+
     def near(self, x, y, max_distance=None):
         """ Return documents near the given point
         """
@@ -132,7 +135,7 @@ class QueryField(object):
             expr[self]['$maxDistance'] = max_distance
         # if bucket_size is not None:
         #     expr['$bucketSize'] = max_distance
-        return QueryExpression(expr)        
+        return QueryExpression(expr)
 
     def near_sphere(self, x, y, max_distance=None):
         """ Return documents near the given point using sphere distances
@@ -142,10 +145,10 @@ class QueryField(object):
         }
         if max_distance is not None:
             expr[self]['$maxDistance'] = max_distance
-        return QueryExpression(expr)        
+        return QueryExpression(expr)
 
     def within_box(self, corner1, corner2):
-        """ 
+        """
             Adapted from the Mongo docs::
 
                 session.query(Places).filter(Places.loc.within_box(cornerA, cornerB)
@@ -156,7 +159,7 @@ class QueryField(object):
                 }}
             })
     def within_radius(self, x, y, radius):
-        """ 
+        """
             Adapted from the Mongo docs::
 
                 session.query(Places).filter(Places.loc.within_radius(1, 2, 50)
@@ -167,7 +170,7 @@ class QueryField(object):
                 }}
             })
     def within_radius_sphere(self, x, y, radius):
-        """ 
+        """
             Adapted from the Mongo docs::
 
                 session.query(Places).filter(Places.loc.within_radius_sphere(1, 2, 50)
@@ -178,7 +181,7 @@ class QueryField(object):
                 }}
             })
     def within_polygon(self, polygon):
-        """ 
+        """
             Adapted from the Mongo docs::
 
                 polygonA = [ [ 10, 20 ], [ 10, 40 ], [ 30, 40 ], [ 30, 20 ] ]
@@ -193,15 +196,15 @@ class QueryField(object):
             })
 
     def in_(self, *values):
-        ''' A query to check if this query field is one of the values 
+        ''' A query to check if this query field is one of the values
             in ``values``.  Produces a MongoDB ``$in`` expression.
         '''
         return QueryExpression({
             self : { '$in' : [self.get_type().wrap_value(value) for value in values] }
         })
-    
+
     def nin(self, *values):
-        ''' A query to check if this query field is not one of the values 
+        ''' A query to check if this query field is not one of the values
             in ``values``.  Produces a MongoDB ``$nin`` expression.
         '''
         return QueryExpression({
@@ -212,86 +215,86 @@ class QueryField(object):
         ''' Create a MongoDB query to check if a field exists on a Document.
         '''
         return QueryExpression({self: {'$exists': exists}})
-    
+
     def __str__(self):
         return self.get_absolute_name()
-    
+
     def __repr__(self):
         return 'QueryField(%s)' % str(self)
-    
+
     def __hash__(self):
         return hash(self.__cached_id)
-    
+
     def __eq__(self, value):
         return self.eq_(value)
     def eq_(self, value):
-        ''' Creates a query expression where ``this field == value`` 
-        
+        ''' Creates a query expression where ``this field == value``
+
             .. note:: The prefered usage is via an operator: ``User.name == value``
         '''
         if isinstance(value, QueryField):
             return self.__cached_id == value.__cached_id
         return QueryExpression({ self : self.get_type().wrap_value(value) })
-    
+
     def __lt__(self, value):
         return self.lt_(value)
     def lt_(self, value):
-        ''' Creates a query expression where ``this field < value`` 
-        
+        ''' Creates a query expression where ``this field < value``
+
             .. note:: The prefered usage is via an operator: ``User.name < value``
         '''
         return self.__comparator('$lt', value)
-    
+
     def __le__(self, value):
         return self.le_(value)
     def le_(self, value):
-        ''' Creates a query expression where ``this field <= value`` 
-        
+        ''' Creates a query expression where ``this field <= value``
+
             .. note:: The prefered usage is via an operator: ``User.name <= value``
         '''
         return self.__comparator('$lte', value)
-    
+
     def __ne__(self, value):
         return self.ne_(value)
     def ne_(self, value):
-        ''' Creates a query expression where ``this field != value`` 
-        
+        ''' Creates a query expression where ``this field != value``
+
             .. note:: The prefered usage is via an operator: ``User.name != value``
         '''
         if isinstance(value, QueryField):
             return self.__cached_id != value.__cached_id
         return self.__comparator('$ne', value)
-    
+
     def __gt__(self, value):
         return self.gt_(value)
     def gt_(self, value):
-        ''' Creates a query expression where ``this field > value`` 
-        
+        ''' Creates a query expression where ``this field > value``
+
             .. note:: The prefered usage is via an operator: ``User.name > value``
         '''
         return self.__comparator('$gt', value)
-    
+
     def __ge__(self, value):
         return self.ge_(value)
     def ge_(self, value):
-        ''' Creates a query expression where ``this field >= value`` 
-        
+        ''' Creates a query expression where ``this field >= value``
+
             .. note:: The prefered usage is via an operator: ``User.name >= value``
         '''
         return self.__comparator('$gte', value)
-    
+
     def elem_match(self, value):
         ''' This method does two things depending on the context:
 
-            1. In the context of a query expression it: 
+            1. In the context of a query expression it:
 
             Creates a query expression to do an $elemMatch on the selected
             field.  If the type of this field is a DocumentField the value
             can be either a QueryExpression using that Document's fields OR
             you can use a dict for raw mongo.
 
-            See the mongo documentation for thorough treatment of 
-            elemMatch: 
+            See the mongo documentation for thorough treatment of
+            elemMatch:
             http://docs.mongodb.org/manual/reference/operator/elemMatch/
 
             2. In the context of choosing fields in a query.fields() expr:
@@ -319,7 +322,7 @@ class QueryField(object):
 
     def exclude(self):
         ''' Use in a query.fields() expression to say this field should be
-            excluded.  The default of fields() is to include only 
+            excluded.  The default of fields() is to include only
             fields which are specified. This allows retrieving of "every field
             except 'foo'".
         '''
@@ -333,62 +336,62 @@ class QueryField(object):
             }
         })
 
-    
+
 class QueryExpression(object):
-    ''' A QueryExpression wraps a dictionary representing a query to perform 
-        on a mongo collection.  The 
-    
+    ''' A QueryExpression wraps a dictionary representing a query to perform
+        on a mongo collection.  The
+
         .. note:: There is no ``and_`` expression because multiple expressions
             can be specified to a single call of :func:`Query.filter`
     '''
     def __init__(self, obj):
         self.obj = obj
     def not_(self):
-        ''' Negates this instance's query expression using MongoDB's ``$not`` 
+        ''' Negates this instance's query expression using MongoDB's ``$not``
             operator
-            
+
             **Example**: ``(User.name == 'Jeff').not_()``
-            
-            .. note:: Another usage is via an operator, but parens are needed 
+
+            .. note:: Another usage is via an operator, but parens are needed
                 to get past precedence issues: ``~ (User.name == 'Jeff')``
             '''
         ret_obj = {}
-        for k, v in self.obj.iteritems():
+        for k, v in self.obj.items():
             if not isinstance(v, dict):
                 ret_obj[k] = {'$ne' : v }
                 continue
             num_ops = len([x for x in v if x[0] == '$'])
             if num_ops != len(v) and num_ops != 0:
                 raise BadQueryException('$ operator used in field name')
-            
+
             if num_ops == 0:
                 ret_obj[k] = {'$ne' : v }
                 continue
-            
-            for op, value in v.iteritems():
+
+            for op, value in v.items():
                 k_dict = ret_obj.setdefault(k, {})
                 not_dict = k_dict.setdefault('$not', {})
                 not_dict[op] = value
-        
-            
+
+
         return QueryExpression(ret_obj)
-    
+
     def __invert__(self):
         return self.not_()
-    
+
     def __or__(self, expression):
         return self.or_(expression)
-    
+
     def or_(self, expression):
-        ''' Adds the given expression to this instance's MongoDB ``$or`` 
+        ''' Adds the given expression to this instance's MongoDB ``$or``
             expression, starting a new one if one does not exst
-            
+
             **Example**: ``(User.name == 'Jeff').or_(User.name == 'Jack')``
-            
+
             .. note:: The prefered usageis via an operator: ``User.name == 'Jeff' | User.name == 'Jack'``
-            
+
             '''
-        
+
         if '$or' in self.obj:
             self.obj['$or'].append(expression.obj)
             return self
@@ -417,7 +420,7 @@ def flatten(obj):
     if not isinstance(obj, dict):
         return obj
     ret = {}
-    for k, v in obj.iteritems():
+    for k, v in obj.items():
         if not isinstance(k, basestring):
             k = str(k)
         if isinstance(v, dict):
@@ -426,4 +429,3 @@ def flatten(obj):
             v = [flatten(x) for x in v]
         ret[k] = v
     return ret
-
